 /*
  * helpers.c
  * Copyright (C) Aitor Cuadrado Zubizarreta <aitor_czr@gnuinos.org>
  * 
  * simple-netaid is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
  * Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  * 
  * simple-netaid is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License along
  * with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * See the COPYING file.
  */

#ifndef _GNU_SOURCE
	#define _GNU_SOURCE 1
#endif

#include "helpers.h"
#include "sbuf.h"
#include "interfaces.h"
#include "ipaddr.h"
#include "wireless.h"
#include "util.h"
#include "def.h"

#define MAX_SIZE 128

void print_active_wifis(const char *wireless_device)
{
	struct sbuf s;
	sbuf_init(&s);	
	scan_active_wifis(wireless_device, &s);
	printf("%s\n", s.buf);
	free(s.buf);
}

short ifquery(const char *ifname)
{
	FILE *fp;
	short state;
	struct sbuf cmd;
		
	sbuf_init(&cmd);
	sbuf_concat(&cmd, 2, "/sbin/ifquery --state ", ifname);
	fp = popen(cmd.buf, "r");
	if(!fp) {
		state=-1;
		goto Free;
	}
	
	state=0;
	if(fgetc(fp)!=EOF) state=1;
	
	pclose(fp);
	
Free:
	free(cmd.buf);
	
	return state;
}

void ifup(const char *ifname)
{
	if(ifquery(ifname)) return;
	struct sbuf cmd;	
	sbuf_init(&cmd);
	sbuf_concat(&cmd, 3, "/sbin/ifup ", ifname, " > /dev/null");
	system(cmd.buf);
	free(cmd.buf);
}

void ifdown(const char *ifname)
{
	if(!ifquery(ifname)) return;
	struct sbuf cmd;	
	sbuf_init(&cmd);
	sbuf_concat(&cmd, 3, "/sbin/ifdown ", ifname, " > /dev/null");
	system(cmd.buf);
	free(cmd.buf);
}

void kill_all_processes()
{
	const char *processes[5] = { "dhclient", "udhcpc", "pump", "dhcp6c", "wpa_supplicant" };
	
	struct dirent *ent;
    
    DIR *proc_dir = opendir("/proc");
    while ((ent = readdir(proc_dir))) {
		
		// Be sure it's a pid:
		if ((*ent->d_name>'0') && (*ent->d_name<='9')) {
			
			pid_t pid = atoi(ent->d_name);
			
			char statFileName[128];
			char *mypid = NULL;
			
			if (asprintf(&mypid, "%jd", (intmax_t) pid) != -1) {
				sprintf(statFileName, "/proc/%s/stat", mypid);
				free(mypid);
				
				int i=0;
				FILE *fp;

Try:				
				{
					fp = fopen(statFileName, "r");
					if(!fp) goto Catch;					
				}
Catch:				
				{
					i++;
					if (i>10 || fp) goto Continue;
					else goto Try;
				}
Continue:		
				if (!fp) {
					//printf("Enable to open the %s file.\n", statFileName);
				} else {
						
					char buffer[512];
					fgets(buffer, 512, fp);
					fclose(fp);
					
					char* cstart=strchr(buffer, '(');
					char* cend  =strrchr(buffer, ')');
					size_t namesize = (cend-cstart<33)?cend-cstart-1:32;
			
					char name[512];
					strncpy(name, cstart+1, namesize);
					name[namesize]='\0';
					if ( (cstart == NULL) || (cend == NULL) ) printf(_("Enable to get the name of the process from %s.\n"), statFileName);
					else {
						for(unsigned int i=0; i<5; i++) {							 
							if(!strcmp(processes[i], name)) {
								if( (kill(pid, SIGTERM)) != 0 ) kill(pid, SIGKILL);
							}
						} // for
					}
				}					
			}
		}
		
	} // while
	
    closedir(proc_dir);	
}

void disconnect(const char *ifname)
{	
	ifdown(ifname);
	interface_down(ifname);
	interface_up(ifname);
}

void wired_connection(const char *ifname)
{	
	kill_all_processes();
	ipaddr_flush(ifname);
	interface_down(ifname);
	ifdown(ifname);
	interface_up(ifname);
	sleep(1);
	ifup(ifname);
}

void wireless_connection(const char *ifname, const char *essid, const char *password, const char *filename)
{
	struct sbuf s;
	sbuf_init(&s);
	
	if(filename) sbuf_addstr(&s, filename);
	else {
		sbuf_addstr(&s, "/tmp/simple-netaid_");
		sbuf_random(&s, 5);
	}
	
	kill_all_processes();
	ipaddr_flush(ifname);
	interface_down(ifname);
	ifdown(ifname);
	interface_up(ifname);
	wpa_passphrase(essid, password, s.buf);
	wpa_supplicant(ifname, s.buf);
	sleep(1);
	ifup(ifname);
	
	if(filename) {
		const char *str=s.buf;
		sbuf_init(&s);
		sbuf_concat(&s, 3, "chmod -R 644 \"", str, "\" > /dev/null");
		system(s.buf);
	} else {
		remove(s.buf);
		if(!file_exists(s.buf)) printf(_("Error deleting %s file.\n"), s.buf);
	}
	
	free(s.buf);		
}

void wpa_passphrase(const char *essid, const char *password, const char *filename)
{	
	if(*password!=0) {
		struct sbuf s;
		sbuf_init(&s);
		sbuf_concat(&s, 7, "/usr/bin/wpa_passphrase \"", essid, "\" \"", password, 
					"\"  | sudo tee -a \"", filename, "\" > /dev/null" );
		system(s.buf);
		free(s.buf);
	} else {
		FILE *fp;	
		fp = fopen(filename, "w");
		if(!fp) {
			printf(_("Cannot open the \"%s\" config file.\n"), filename);
			exit(EXIT_FAILURE);
		}
		
		fprintf( fp, "network={\n\tssid=\"%s\"\n\tkey_mgmt=NONE\n}", essid);
		fclose(fp);
	}
}

void wpa_supplicant(const char *ifname, const char *filename)
{
	struct sbuf cmd;	
	sbuf_init(&cmd);
	sbuf_concat(&cmd, 5, "/sbin/wpa_supplicant -B -i\"", ifname, "\" -c\"", filename, "\"  > /dev/null");
	system(cmd.buf);
	free(cmd.buf);
}

