 /*
  * iproute.c
  * Copyright (C) Aitor Cuadrado Zubizarreta <aitor_czr@gnuinos.org>
  * 
  * Most of the code of this file is taken from the RTnetlink service routines
  * of the iproute2 project.
  * 
  * https://git.kernel.org/pub/scm/network/iproute2/iproute2.git/
  * 
  * simple-netaid is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
  * Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  * 
  * simple-netaid is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License along
  * with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * See the COPYING file.
  */

#include <stdbool.h>  
#include "iproute.h"
#include "ll_map.h"
#include "def.h"

#define IFNAME_SIZ 64

static struct
{
	unsigned int tb;
	int cloned;
	char *flushb;
	int oif, oifmask;
	inet_prefix rvia;
	inet_prefix mdst;
	inet_prefix msrc;
} filter;

static char ifname[IFNAME_SIZ];	
static struct rtnl_handle rtnl_h = { .fd = -1 };

char* iproute()
{	
	if (rtnl_open(&rtnl_h, 0) < 0) exit(1);
	
	int do_ipv6 = AF_UNSPEC;	
	rtnl_filter_t filter_fn;
	
	memset(&ifname, 0, IFNAME_SIZ);
		
	filter_fn = (rtnl_filter_t)print_route;
	
	memset(&filter, 0, sizeof(filter));
	filter.mdst.bitlen = -1;
	filter.msrc.bitlen = -1;
	filter.oif = 0;
	if (filter.oif > 0)
		filter.oifmask = -1;
	filter.tb = RT_TABLE_MAIN;
	
	if (filter.tb) do_ipv6 = AF_INET;
	
	if (!filter.cloned) {
		if (rtnl_wilddump_request(&rtnl_h, do_ipv6, RTM_GETROUTE) < 0) {
			perror(_("Cannot send dump request"));
			rtnl_close(&rtnl_h);
			exit(1);
		}
	} else {
		if (rtnl_rtcache_request(&rtnl_h, do_ipv6) < 0) {
			perror(_("Cannot send dump request"));
			rtnl_close(&rtnl_h);
			exit(1);
		}
	}

	if (rtnl_dump_filter_nc(&rtnl_h, filter_fn, stdout, 0) < 0) {
		fprintf(stderr, _("Dump terminated\n"));
		rtnl_close(&rtnl_h);
		exit(1);
	}

	rtnl_close(&rtnl_h);
	
	free(filter.flushb);
	
	return ifname;
}

rtnl_filter_t print_route(const struct sockaddr_nl *who, struct nlmsghdr *n)
{
	struct rtmsg *r = NLMSG_DATA(n);
	int len = n->nlmsg_len;
	struct rtattr *tb[RTA_MAX+1];
	int host_len;
		
	host_len = af_bit_len(r->rtm_family);
	
	if (n->nlmsg_type != RTM_NEWROUTE && n->nlmsg_type != RTM_DELROUTE) {
		fprintf(stderr, _("Not a route: %08x %08x %08x\n"),
			n->nlmsg_len, n->nlmsg_type, n->nlmsg_flags);
		return (rtnl_filter_t)(-1);
	}
	if (filter.flushb && n->nlmsg_type != RTM_NEWROUTE)
		return 0;
		
	len -= NLMSG_LENGTH(sizeof(*r));
	if (len < 0) {
		fprintf(stderr, _("BUG: wrong nlmsg len %d\n"), len);
		return (rtnl_filter_t)(-1);
	}
   
	int flags=0;		
	if (r->rtm_flags) {
		if (RTNH_F_DEAD)       flags=1;
		if (RTNH_F_ONLINK)     flags=2;
		if (RTNH_F_PERVASIVE)  flags=3;
		if (RTNH_F_OFFLOAD)    flags=4;
		if (RTM_F_NOTIFY)      flags=5;
		if (RTNH_F_LINKDOWN)   flags=6;
		if (RTNH_F_UNRESOLVED) flags=7;
		if (RTM_F_OFFLOAD)     flags=8;
		if (RTM_F_TRAP)        flags=9;
	}
	   
	parse_rtattr(tb, RTA_MAX, RTM_RTA(r), len);
	
	/*  RTA_IIF is the INPUT INTERFACE INDEX
 	 *  RTA_OIF is the OUTPUT INTERFACE INDEX
	 *  For more details, see the manpages of netlink:
     *  http://man7.org/linux/man-pages/man7/rtnetlink.7.html     
     * */
	if (tb[RTA_OIF] && filter.oifmask != -1 && (tb[RTA_GATEWAY] && filter.rvia.bitlen != host_len) && (flags == 0 || flags == 9)) 
		strcpy(ifname, ll_index_to_name(*(int *)RTA_DATA(tb[RTA_OIF])));
	
	return 0;
}

int rtnl_rtcache_request(struct rtnl_handle *rtnl_h, int family)
{
	struct {
		struct nlmsghdr nlh;
		struct rtmsg rtm;
	} req = {
		.nlh.nlmsg_len = sizeof(req),
		.nlh.nlmsg_type = RTM_GETROUTE,
		.nlh.nlmsg_flags = NLM_F_ROOT | NLM_F_REQUEST,
		.nlh.nlmsg_seq = rtnl_h->dump = ++rtnl_h->seq,
		.rtm.rtm_family = family,
		.rtm.rtm_flags = RTM_F_CLONED,
	};
	struct sockaddr_nl nladdr = { .nl_family = AF_NETLINK };

	return sendto(rtnl_h->fd, (void *)&req, sizeof(req), 0, (struct sockaddr *)&nladdr, sizeof(nladdr));
}
