 /*
  * utils.c
  * Copyright (C) Aitor Cuadrado Zubizarreta <aitor_czr@gnuinos.org>
  * 
  * Most of the code of this file is taken from the RTnetlink service routines
  * of the iproute2 project.
  * 
  * https://git.kernel.org/pub/scm/network/iproute2/iproute2.git/
  * 
  * simple-netaid is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
  * Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  * 
  * simple-netaid is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License along
  * with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * See the COPYING file.
  */

#include <assert.h>

#include "util.h"
#include "ll_map.h"
#include "def.h"


int cmdlineno;

// Copy string "in" with at most "insz" chars to buffer "out", which
// is "outsz" bytes long. The output is always 0-terminated. Unlike
// strncpy(), strncpy_t() does not zero fill remaining space in the
// output buffer:
char* strncpy_t(char* out, size_t outsz, const char* in, size_t insz)
{
   assert(outsz > 0);
   while(--outsz > 0 && insz > 0 && *in) { *out++ = *in++; insz--; }
   *out = 0;
   return out;
}

// remove a character from char*
char *strdelch(char *str, char ch)
{
    char *current = str;
    char *tail = str;

    while(*tail)
    {
        if(*tail == ch)
        {
            tail++;
        }
        else
        {
            *current++ = *tail++;
        }
    }
    *current = 0;
    return str;
}

int af_bit_len(int af)
{
   switch (af) {
   case AF_INET6:
      return 128;
   case AF_INET:
      return 32;
   case AF_DECnet:
      return 16;
   case AF_IPX:
      return 80;
   default:
      return 20;
   }

   return 0;
}

int matches(const char *cmd, const char *pattern)
{
   int len = strlen(cmd);
   if (len > strlen(pattern))
      return -1;
   return memcmp(pattern, cmd, len);
}

ssize_t getcmdline(char **linep, size_t *lenp, FILE *in)
{
   ssize_t cc;
   char *cp;

   if ((cc = getline(linep, lenp, in)) < 0)
      return cc;   /* eof or error */
   ++cmdlineno;

   cp = strchr(*linep, '#');
   if (cp)
      *cp = '\0';

   while ((cp = strstr(*linep, "\\\n")) != NULL) {
      char *line1 = NULL;
      size_t len1 = 0;
      ssize_t cc1;

      if ((cc1 = getline(&line1, &len1, in)) < 0) {
         fprintf(stderr, _("Missing continuation line\n"));
         return cc1;
      }

      ++cmdlineno;
      *cp = 0;

      cp = strchr(line1, '#');
      if (cp)
         *cp = '\0';

      *lenp = strlen(*linep) + strlen(line1) + 1;
      *linep = realloc(*linep, *lenp);
      if (!*linep) {
         fprintf(stderr, _("Out of memory\n"));
         *lenp = 0;
         return -1;
      }
      cc += cc1 - 2;
      strcat(*linep, line1);
      free(line1);
   }
   return cc;
}

/* split command line into argument vector */
int makeargs(char *line, char *argv[], int maxargs)
{
   static const char ws[] = " \t\r\n";
   char *cp;
   int argc = 0;

   for (cp = line + strspn(line, ws); *cp; cp += strspn(cp, ws)) {
      if (argc >= (maxargs - 1)) {
         fprintf(stderr, _("Too many arguments to command\n"));
         exit(1);
      }

      /* word begins with quote */
      if (*cp == '\'' || *cp == '"') {
         char quote = *cp++;

         argv[argc++] = cp;
         /* find ending quote */
         cp = strchr(cp, quote);
         if (cp == NULL) {
            fprintf(stderr, _("Unterminated quoted string\n"));
            exit(1);
         }
         *cp++ = 0;
         continue;
      }

      argv[argc++] = cp;
      /* find end of word */
      cp += strcspn(cp, ws);
      *cp++ = 0;
   }
   argv[argc] = NULL;

   return argc;
}

/* This is a necessary workaround for multicast route dumps */
int get_real_family(int rtm_type, int rtm_family)
{
   if (rtm_type != RTN_MULTICAST)
      return rtm_family;

   return rtm_family == RTNL_FAMILY_IPMR ? AF_INET : AF_INET6;
}

int inet_addr_match(const inet_prefix *a, const inet_prefix *b, int bits)
{
   const __u32 *a1 = a->data;
   const __u32 *a2 = b->data;
   int words = bits >> 0x05;

   bits &= 0x1f;

   if (words)
      if (memcmp(a1, a2, words << 2))
         return -1;

   if (bits) {
      __u32 w1, w2;
      __u32 mask;

      w1 = a1[words];
      w2 = a2[words];

      mask = htonl((0xffffffff) << (0x20 - bits));

      if ((w1 ^ w2) & mask)
         return 1;
   }

   return 0;
}

short file_exists(const char *filename)
{
   int fd=open(filename, O_RDONLY);
   if (fd<0) return (errno==ENOENT)?-1:-2;    
   close(fd);
   return 0;
}

